import { AlertService, DialogService, Form, LoadingService, Modal, ModalService, SelectMultiple, TextInput } from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { ITekGridLayout, TekGrid, TekRestDatasource } from '@zeedhi/teknisa-components-common';
import { RichText } from '@zeedhi/zd-richtext-common';
import { Utilities } from '@/util/Utilities';

export class NoticeForOperatorController {
    private addModal!: Modal;

    private viewModal!: Modal;

    public isHTML = 'inline';

    public isNotHTML = 'zd-display-none';

    public isHTMLVisible = true;

    public isLinkVisible = true;

    public isLinkDirVisible = true;

    public reloadGrid() {
        Metadata.getInstance<TekGrid>('notice-grid').reload();
    }

    public async openAddModal() {
        this.addModal = await ModalService.createFromJson('noticeForOperator/addModal/addModal', true);
        this.addModal.show();
        this.setVisibilities({ component: { value: '' } });
    }

    public closeAddModal() {
        this.addModal.hide();
        this.addModal?.destroy();
    }

    public async openViewModal() {
        this.viewModal = await ModalService.createFromJson('noticeForOperator/viewModal/viewModal', true);
        this.viewModal.show();
    }

    public closeViewModal() {
        this.viewModal.hide();
        this.viewModal?.destroy();
    }

    public async addNewNotice() {
        const form = Metadata.getInstance<Form>('add-modal-form');
        if (!form.validate()) return;
        const formData = form.value;
        let content;
        let linkDir = '';
        if (formData.type === 'H') {
            const richText = Metadata.getInstance<RichText>('rich-text');
            content = richText.content;
            if (content === '') {
                DialogService.show(
                    {
                        name: 'nocontent-dialog',
                        type: 'error',
                        title: 'Comunicado sem conteúdo',
                        text: 'O comunicado deve ter conteúdo',
                    },
                );
                return;
            }
        } else if (formData.type === 'I') {
            content = formData.link;
            linkDir = formData.linkDirecionamento;
            if (content === '' || content === null) {
                DialogService.show(
                    {
                        name: 'nocontent-dialog',
                        type: 'error',
                        title: 'Comunicado sem conteúdo',
                        text: 'O comunicado deve ter um link',
                    },
                );
                return;
            }
        } else {
            content = formData.link;
            if (content === '' || content === null) {
                DialogService.show(
                    {
                        name: 'nocontent-dialog',
                        type: 'error',
                        title: 'Comunicado sem conteúdo',
                        text: 'O comunicado deve ter um link',
                    },
                );
                return;
            }
        }
        const requestBody = {
            title: formData.title,
            timeOutNotice: formData.timeOutNotice,
            publicoAlvoNotice: formData.publicoAlvoNotice,
            dateBegin: formData['date-range'][0],
            dateEnd: formData['date-range'][1],
            type: formData.type,
            active: formData.active,
            content,
            linkDir,
            produto: formData.produto,
        };
        LoadingService.show('Adicionando...');
        await Http.post('/notice', requestBody)
            .then(({ data }) => {
                AlertService.show({
                    name: 'add-alert',
                    text: data.success,
                    color: 'success',
                    right: true,
                    top: true,
                });
                this.closeAddModal();
                this.reloadGrid();
            }).catch((error) => {
                AlertService.show({
                    name: 'add-alert',
                    text: error,
                    color: 'error',
                    right: true,
                    top: true,
                });
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public confirmDelete({ event }) {
        event.preventDefault();
        DialogService.show({
            name: 'delete-dialog',
            type: 'warning',
            title: 'Excluir Registro(s)',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                { name: 'no', component: 'ZdButton', label: 'Não', events: { click: () => { DialogService.hide(); } }, outline: true },
                { name: 'yes', component: 'ZdButton', label: 'Sim', events: { click: () => { DialogService.hide(); this.deleteNotice(); } } },
            ],
        });
    }

    public async deleteNotice() {
        const grid = Metadata.getInstance<TekGrid>('notice-grid');
        const rows = grid.selectedRows;
        let ids = '';
        for (let i = 0; i < rows.length; i += 1) {
            if (i === 0) ids = `id[${i}]=${rows[i].NRINFECOMMERCE}`;
            else ids += `&id[${i}]=${rows[i].NRINFECOMMERCE}`;
        }
        LoadingService.show('Excluindo...');
        await Http.delete(`/notice?${ids}`)
            .then(({ data }) => {
                AlertService.show({
                    name: 'delete-alert',
                    text: data.success,
                    color: 'success',
                    right: true,
                    top: true,
                });
                this.reloadGrid();
                grid.selectedRows = [];
            }).catch((error) => {
                AlertService.show({
                    name: 'delete-alert',
                    text: error,
                    color: 'error',
                    right: true,
                    top: true,
                });
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public async fillViewModal({ component, row }: any) {
        if (component.editing === true) return;
        await this.openViewModal();
        const form = Metadata.getInstance<Form>('view-modal-form').value;
        form.id = row.NRINFECOMMERCE;
        form.title = row.DSTITULOINF;
        form['date-range'] = [row.DTINFORMARIVI, row.DTEXPIRAINFO];
        form.type = row.IDNOTIFICACAO;
        form.active = row.IDATIVO;
        form.timeOutNoticeEdit = row.IDTIMEOUT;
        form.publicoAlvoNotice = row.CDPUBLICOALVO;
        if (row.NRPRODUTOS !== null) {
            const nrprodutos = row.NRPRODUTOS.split(',');
            Metadata.getInstance<SelectMultiple>('produtoEdit').setValue(nrprodutos);
        }
        this.setVisibilities({ component: { value: form.type } });
        if (form.type === 'H') {
            const richText = Metadata.getInstance<RichText>('rich-text');
            richText.content = row.TXCONTEUDOINF;
        } else if (form.type === 'I') {
            form.link = row.TXCONTEUDOINF;
            form.linkDirecionamento = row.LINKDIRIMAGE;
        } else {
            form.link = row.TXCONTEUDOINF;
        }
    }

    public setVisibilities({ component }: any) {
        if (component.value === '') {
            this.isHTML = 'inline';
            this.isNotHTML = 'zd-display-none';
            this.isHTMLVisible = false;
            this.isLinkVisible = false;
            this.isLinkDirVisible = false;
        } else if (component.value === 'H') {
            Metadata.getInstance<TextInput>('link').value = null;
            this.isHTML = 'inline';
            this.isNotHTML = 'zd-display-none';
            this.isHTMLVisible = true;
            this.isLinkVisible = false;
            this.isLinkDirVisible = false;
        } else if (component.value === null) {
            Metadata.getInstance<TextInput>('link').value = null;
            this.isHTML = 'none';
            this.isNotHTML = 'zd-display-inline';
            this.isHTMLVisible = false;
            this.isLinkVisible = false;
            this.isLinkDirVisible = false;
        } else if (component.value === 'I') {
            Metadata.getInstance<TextInput>('link').value = null;
            this.isHTML = 'none';
            this.isNotHTML = 'zd-display-inline';
            this.isHTMLVisible = false;
            this.isLinkVisible = true;
            this.isLinkDirVisible = true;
        } else {
            Metadata.getInstance<TextInput>('link').value = null;
            this.isHTML = 'none';
            this.isNotHTML = 'zd-display-inline';
            this.isHTMLVisible = false;
            this.isLinkVisible = true;
            this.isLinkDirVisible = false;
        }
    }

    public async cancelEditingInGrid({ component, event }: any) {
        event.preventDefault();
        const grid = component;
        await grid.cancelEditedRows();
    }

    public async updateInModal() {
        const formStructure = Metadata.getInstance<Form>('view-modal-form');
        if (!formStructure.validate()) return;
        const form = formStructure.value;
        const requestBody = [] as any;
        const content = form.type === 'H'
            ? Metadata.getInstance<RichText>('rich-text').content
            : form.link;
        if (content === '') {
            DialogService.show(
                {
                    name: 'nocontent-dialog',
                    type: 'error',
                    title: 'Comunicado sem conteúdo',
                    text: 'O comunicado deve ter conteúdo',
                },
            );
            return;
        }
        requestBody.push({
            id: form.id,
            title: form.title,
            dateBegin: form['date-range'][0],
            dateEnd: form['date-range'][1],
            type: form.type,
            active: form.active,
            content,
            timeOutNoticeEdit: form.timeOutNoticeEdit,
            publicoAlvoNoticeEdit: form.publicoAlvoNotice,
            linkDir: form.linkDirecionamento,
            produtoEdit: form.produtoEdit,
        });
        await this.updateNotice(requestBody, Metadata.getInstance<TekGrid>('notice-grid'), formStructure);
    }

    public async updateInGrid({ component, event }: any) {
        event.preventDefault();
        const grid = component as TekGrid;
        const modifiedItems = grid.getEditedRows();
        const requestBody = [] as any;
        modifiedItems.forEach((item) => {
            requestBody.push({
                id: item.NRINFECOMMERCE,
                title: item.DSTITULOINF,
                dateBegin: item.DTINFORMARIVI,
                dateEnd: item.DTEXPIRAINFO,
                type: item.IDNOTIFICACAO,
                active: item.IDATIVO,
                content: item.TXCONTEUDOINF,
                timeOutNoticeEdit: item.IDTIMEOUT,
                publicoAlvoNoticeEdit: item.CDPUBLICOALVO,
                linkDir: item.LINKDIRIMAGE,
                produtoEdit: null,
            });
        });
        await this.updateNotice(requestBody, grid, null);
    }

    private updateNotice(requestBody: [], grid: TekGrid, form: Form | null) {
        LoadingService.show('Atualizando...');
        Http.put('/notice', requestBody)
            .then(({ data }) => {
                AlertService.show({
                    name: 'update-alert',
                    text: data.success,
                    color: 'success',
                    right: true,
                    top: true,
                });
                this.reloadGrid();
                grid.cancelEditedRows();

                if (form !== null) this.closeViewModal();
            }).catch((error) => {
                AlertService.show({
                    name: 'update-alert',
                    text: error,
                    color: 'error',
                    right: true,
                    top: true,
                });
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public addTimeOut() {
        const grid = Metadata.getInstance<TekGrid>('timeout-grid');
        const newId = 0;
        grid.addNewRow({ ID: newId }, 'start');
    }

    public saveTimeOutNotice({ component }: IEventParam<TekGrid>) {
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/saveTimeOutNotice', {
                ROW: component.getEditedRows(),
            }).then(() => {
                Utilities.showSuccessMessage();
                component.datasource.get();
            }).catch((error) => {
                if (error.response.data.error === 'EXISTE') {
                    Utilities.showDialogMessage('error', 'Operação bloqueada!', 'O Nome informado já está cadastrado.');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessageTimeOutNotice({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteTimeOutNotice();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteTimeOutNotice() {
        const grid = Metadata.getInstance<TekGrid>('timeout-grid');
        LoadingService.show();
        Http.post('/deleteTimeOutNotice', grid.selectedRows)
            .then(() => {
                grid.selectedRows = [];
                grid.reload();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public getColumnStyle(params: any) {
		return params.row.IDATIVO === 'N' ? 'om-grid-column-inactive' : '';
	}

    public async filterOnMounted() {
        const noticeForOperadorGrid = Metadata.getInstance<TekGrid>('notice-grid');
		if (noticeForOperadorGrid.datasource instanceof TekRestDatasource) {
			const dynamicFilter = {
				IDATIVO: [
					{
						operation: 'EQUALS',
						relation: 'AND',
						value: 'S',
					},
				],
			};
			const defaultLayout: ITekGridLayout = {
				name: 'Padão',
				dynamicFilter,
			};
			noticeForOperadorGrid.layoutOptions?.updateDefaultLayout(defaultLayout);
		}
    }
}
